# MTCodeBeautifier Movable Type Plugin
# Copyright  2003 Sean Voisen.
# http://voisen.org
# sean@voisen.org
#
# Beautifier is  Mike Jewell.
# For more information on Beautifier, please visit
# http://www.beautifier.org
#
# Modification added by Samuel Neff
# to support ColdFusion, 6/15/03
# http://www.rewindlife.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package Voisen::CodeBeautifier;

use MT;
use MT::Template::Context;
use MT::Util qw/decode_html/;
use CGI qw/:standard/;

MT::Template::Context->add_container_tag(
	CodeBeautifier => sub {
		&beautifier;
	}
);

sub highlight_php3
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_php3;
	my $hf = new HFile::HFile_php3;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}

sub highlight_perl
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_perl;
	my $hf = new HFile::HFile_perl;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}

sub highlight_scheme
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_scheme;
	my $hf = new HFile::HFile_scheme;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}

sub highlight_java
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_java;
	my $hf = new HFile::HFile_java;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}

sub highlight_as
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_actionscript;
	my $hf = new HFile::HFile_actionscript;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}

#
# Modification added by Samuel Neff
# to support ColdFusion, 6/15/03
#
sub highlight_cf
{
	require Beautifier::Core;
	require Output::HTML;
	require HFile::HFile_coldfusion;
	my $hf = new HFile::HFile_coldfusion;
	$hf->{notrim} = 1;
	my $highlighter = new Beautifier::Core($hf, new Output::HTML);
	return $highlighter->highlight_text($_[0]);
}


sub beautifier
{
	my ($ctx, $args, $cond) = @_;
	my $result = '';
	my $builder = $ctx->stash('builder');
	my $tokens = $ctx->stash('tokens');
	
	my $language = build_expr($ctx, $args->{language}, $cond);
	
	# Case insensitive
	$language = lc( $language );
	
	if( $language eq "actionscript" || $language eq "as" )
	{
		$result = highlight_as($builder->build($ctx, $tokens));
	}
	elsif( $language eq "php3" || $language eq "php" )
	{
		$result = highlight_php3($builder->build($ctx, $tokens));
	}
	elsif( $language eq "perl" )
	{
		$result = highlight_perl($builder->build($ctx, $tokens));
	}
	elsif( $language eq "java" )
	{
		$result = highlight_java($builder->build($ctx, $tokens));
	}
	elsif( $language eq "scheme" )
	{
		$result = highlight_scheme($builder->build($ctx, $tokens));
	}

#
# Modification added by Samuel Neff
# to support ColdFusion, 6/15/03
#
	elsif( $language eq "coldfusion" || $language eq "cf" )
	{
		$result = highlight_cf($builder->build($ctx, $tokens));
	}
	else
	{
		# This is an unsupported language, so just return the tokens
		$result = $builder->build($ctx, $tokens);
	}
	return $result;
}

# Taken from Brad Choate's MTMacro plug-in
# Allows functionality with that plug-in
sub build_expr 
{
    my ($ctx, $val, $cond) = @_;
    $val = decode_html($val);
    if (($val =~ m/\<MT.*?\>/) || ($val =~ s/\[(\/?MT(.*?))\]/\<$1\>/g)) 
	{
		my $builder = $ctx->stash('builder');
		my $tok = $builder->compile($ctx, $val);
		defined($val = $builder->build($ctx, $tok, $cond))
		  or return $ctx->error($builder->errstr);
    }
    return $val;
}

1;
